SystemOrganization addCategory: #'Bindings-Base'!
SystemOrganization addCategory: #'Bindings-Tests'!
SystemOrganization addCategory: #'Bindings-Widgets'!

TextMorph subclass: #BindableTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Widgets'!

!BindableTextMorph methodsFor: 'accessing-generated' stamp: 'topa 1/18/2010 20:49'!
contents
	"Generated automatically!!"

	<bindable>
	^ self valueOfVariableNamed: #text! !

!BindableTextMorph methodsFor: 'accessing-generated' stamp: 'topa 1/18/2010 20:55'!
contents: anObject
	"Generated automatically!!"

	<bindable>
	(anObject = (self contents))
		ifFalse: [
			self bindings triggerEvent: #aboutToChange with: (self -> #contents).
			self contentsAsIs: (anObject ifNil: ['']).
			self bindings triggerEvent: #hasChanged with: (self -> #contents)].
! !

!Morph methodsFor: '*bindings' stamp: 'topa 1/18/2010 16:11'!
bindableOrFirstComponent: localKeyPath

	^ localKeyPath isString 
		ifTrue: [localKeyPath]
		ifFalse: [localKeyPath first]! !

!Morph methodsFor: '*bindings' stamp: 'topa 1/26/2010 10:38'!
ensureManagedCollection: bindable

	| mc |
	mc := self instVarNamed: bindable
		ifAbsent: [self managedCollectionInExtenstion: bindable].
			
	(mc isNil or: [mc isManagedCollection not])
		"we rely on the fact that if the bindable is no InstVar, 
		#managedCollectionInExtenstion: will return an instance of ManagedCollection"
		ifTrue: [
			mc := (self class managedCollectionClass wrapping: mc).
			self instVarNamed: bindable put: mc].
	^ mc! !

!Morph methodsFor: '*bindings' stamp: 'topa 1/12/2010 08:22'!
managedCollectionInExtenstion: bindable

	| propName |
	propName :=  (bindable, ' managedCollection').
	
	(self assureExtension hasProperty: propName) 
		ifFalse: [self assureExtension 
			setProperty: propName 
			toValue: (self class managedCollectionClass wrapping: (self -> bindable))].
		
	^ self assureExtension valueOfProperty: propName! !

Collection subclass: #ManagedCollection
	instanceVariableNames: 'actualCollection selection managedType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Base'!

ManagedCollection class
	instanceVariableNames: 'knownMutators'!

ManagedCollection class
	instanceVariableNames: 'knownMutators'!

!ManagedCollection class methodsFor: 'accessing' stamp: 'topa 1/18/2010 14:04'!
knownMutators

	knownMutators ifNil: [
		self knownMutators: 
			(IdentitySet withAll: (
				{#atNewIndex:put: . #at:put: . #at:ifAbsentPut: }, "the indexed ones"
				{#associationDeclareAt: . #member . #removeKey: . #removeKey:ifAbsent: . #noCheckNoGrowFillFrom:}, "unordered specials"
				{#add:after: . #add:afterIndex: . #add:before: . #add:beforeIndex: . #addFirst: . #addLast: . 
				 #addAllFirst: . #addAllFirstUnlessAlreadyPresent: . #addAllLast: . #insert:before: . 
				 #removeFirst . #removeFirst: . #removeLast . #removeLast: . #removeAt: .  #removeIndex: .
				 #grow . #setCollection: . #setContents: . #reverseInPlace }, "sequencable friends"
				{#sortTopologically . #reSort  . #sort:to: } "sorters"
			))].
	^ knownMutators! !

!ManagedCollection class methodsFor: 'accessing' stamp: 'topa 1/11/2010 22:06'!
knownMutators: aCollection

	knownMutators := aCollection.! !

!ManagedCollection class methodsFor: 'as yet unclassified' stamp: 'topa 1/11/2010 22:36'!
wrapping: aCollection

	^ self new actualCollection: aCollection! !

!ManagedCollection methodsFor: 'private' stamp: 'topa 2/9/2010 10:26'!
actualCollection

	^ actualCollection! !

!ManagedCollection methodsFor: 'private' stamp: 'topa 2/9/2010 10:26'!
actualCollection: aCollection

	actualCollection :=  aCollection.! !

!ManagedCollection methodsFor: 'changing' stamp: 'topa 2/9/2010 10:35'!
add: anObject
	"notifies bindees that anyone has changed collection
	and forwards message to inner collection"

	| ret changedescriptor |
	
	changedescriptor := ChangeDescriptorMessage new
			operation: #add;
			keyPathOfChange: #();
			lastSender: self;
			changedValue: anObject.

	self bindings triggerEvent: #aboutToChange with: changedescriptor.
	ret := self actualCollection add: anObject.
	self bindings triggerEvent: #hasChanged with: changedescriptor.
	^ ret! !

!ManagedCollection methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:36'!
arrangedObjects
	"the arrangedobjects somebody can bind to (mapped to the actual collection)"

	<bindableAsCollection>
	self bindings triggerEvent: #accessed with: (self -> #arrangedObjects).
	^ self actualCollection! !

!ManagedCollection methodsFor: 'changing' stamp: 'topa 2/9/2010 10:32'!
at: aKey put: anObject
	"notifies bindees that anyone has changed collection
	and forwards message to inner collection"

	self flag: #NeedsReview.
	^ (self actualCollection respondsTo: #at:put:)
		ifTrue: [ | ret |
			self bindings triggerEvent: #aboutToChange with: anObject.
			"unregister old object, if present"
			ret := self actualCollection at: aKey put: anObject.
			"register new object"
			self bindings triggerEvent: #hasChanged with: anObject.
			ret]
		ifFalse: [self actualCollection at: aKey put: anObject]
! !

!ManagedCollection methodsFor: 'changing' stamp: 'topa 2/9/2010 10:32'!
do: aBlock
	"notifies bindees that anyone has accessed collection
	and forwards message to inner collection"

	self flag: #NeedsReview.
	self bindings triggerEvent: #accessed with: (self -> #arrangedObjects).
	^ self actualCollection do: aBlock.
! !

!ManagedCollection methodsFor: 'error handling' stamp: 'topa 2/9/2010 10:36'!
doesNotUnderstand: aMessage
	"forwards the not understood message to the inner collection.
	in the case that something is changed, this method calls an event hasChanged:"

	self flag: #NeedsReview.
	(self class knownMutators includes: aMessage selector)
		ifTrue: [
			| result |	
			self bindings triggerEvent: #aboutToChange with: (self actualCollection "????" -> aMessage selector).
			result := self actualCollection perform: aMessage selector withArguments: aMessage arguments.
			self bindings triggerEvent: #hasChanged with: (self actualCollection "????" -> aMessage selector).
			^ result	].
	^ self actualCollection perform: aMessage selector withArguments: aMessage arguments ! !

!ManagedCollection methodsFor: 'as yet unclassified' stamp: 'AK 2/6/2010 12:40'!
innerbind: keypath withProperty: property as: bindingtype
"Binds the inner bindables (prepared objects) to the managedcollection parent recursively by stepping and cutting key path.
Does not cut anything of the keypath itself but forwards it to the bound objects."

	((keypath first == #arrangedObjects) and: [keypath size == 0])
		ifTrue: [ "do nothing, because just needs to manage its objects" ].
		
	((keypath first == #arrangedObjects) and: [keypath size > 0])
		ifTrue: [ self actualCollection do: [ :eachobject |
				(self bindings)
					watchInnerKey: keypath
					as: bindingtype
					notifying: eachobject
					with: keypath allButFirst.
				eachobject innerbind: keypath allButFirst withProperty: property as: bindingtype ]].
		
	((keypath first == #selection) and: [keypath size == 0])
		ifTrue: [ "should not be empty" Error new ].
		
	((keypath first == #selection) and: [keypath size > 0])
		ifTrue: [ selection notNil ifTrue: [
			(self bindings)
				watchInnerKey: keypath
				as: bindingtype
				notifying: selection
				with: keypath allButFirst.
			(self selection binding)
				watchInnerKey: keypath allButFirst
				as: bindingtype
				notifying: self
				with: keypath.
			selection innerbind: keypath withProperty: property as: bindingtype ]].

! !

!ManagedCollection methodsFor: 'as yet unclassified' stamp: 'AK 2/6/2010 18:43'!
isManagedCollection
	^true! !

!ManagedCollection methodsFor: 'changing' stamp: 'topa 2/9/2010 10:35'!
remove: anObject ifAbsent: aBlock
	"notifies bindees that anyone has changed collection
	and forwards message to inner collection"

	| ret changedescriptor |
	
	changedescriptor := ChangeDescriptorMessage new
			operation: #remove;
			keyPathOfChange: #();
			lastSender: self;
			changedValue: anObject.
	
	self bindings triggerEvent: #aboutToChange with: changedescriptor.
	ret := self actualCollection remove: anObject ifAbsent: aBlock.
	self bindings triggerEvent: #hasChanged with: changedescriptor.
	^ ret! !

!ManagedCollection methodsFor: 'accessing' stamp: 'AK 1/31/2010 15:02'!
selection
"the selection somebody can bind to (mapped to an inner variable)"

	<bindable>
	^ self valueOfVariableNamed: #selection! !

!ManagedCollection methodsFor: 'accessing' stamp: 'AK 1/31/2010 15:03'!
selection: anObject
"the selection somebody can bind to (mapped to an inner variable)"

	<bindable>
	self setVariableNamed: #selection toValue: anObject.! !

PluggableListMorph subclass: #SelectableListMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Widgets'!

!SelectableListMorph methodsFor: 'accessing compatibility' stamp: 'topa 1/18/2010 18:51'!
changeModelSelection: anInteger

	^ self selectionIndex: anInteger! !

!SelectableListMorph methodsFor: 'accessing compatibility' stamp: 'topa 1/18/2010 11:02'!
getList

	^ self list! !

!SelectableListMorph methodsFor: 'accessing compatibility' stamp: 'topa 1/18/2010 11:07'!
getListSize

	^self getList ifNil: [0] ifNotNil: [:list | list size]! !

!SelectableListMorph methodsFor: 'accessing' stamp: 'topa 1/18/2010 17:09'!
list

	<bindableAsCollection>
	^ (self valueOfVariableNamed: #list) ifNil: [self list: OrderedCollection new. self list]! !

!SelectableListMorph methodsFor: 'accessing' stamp: 'topa 1/17/2010 20:23'!
list: anObject

	<bindableAsCollection>
	self setVariableNamed: #list toValue: anObject.
	self updateList.
	! !

!SelectableListMorph methodsFor: 'selection' stamp: 'topa 2/9/2010 18:36'!
selection

	<bindable>
	"bindings stub"
	| ret |
	self selectionIndex = 0 ifTrue: [ ^nil ].
	ret := list 
		ifNotNil: [list at: self selectionIndex ]
		ifNil: [self getListItem: self selectionIndex].
	self bindings triggerEvent: #accessed with: (AccessDescriptorMessage new
		accessedKeyPath: #(selection);
		lastSender: self;
		accessedValue: ret).
	^ ret! !

!SelectableListMorph methodsFor: 'selection' stamp: 'topa 2/9/2010 18:29'!
selection: item
	"Called from outside to request setting a new selection."

	<bindable>
	| selectionDesc |
	"bindings stub"
	(self selectionIndex ~= 0 and: [(item = (self getListItem: self selectionIndex))]) ifTrue: [^ self].
	selectionDesc := ChangeDescriptorMessage new
		operation: #selection; keyPathOfChange: #(); lastSender: self; changedValue: item.
	self bindings triggerEvent: #aboutToChange with: selectionDesc.
	self selectionIndex: (self getList indexOf: item).
	self bindings triggerEvent: #hasChanged with: selectionDesc.
! !

!SelectableListMorph methodsFor: 'selection' stamp: 'topa 2/9/2010 18:36'!
selectionIndex

	<bindable>
	| ret |
	ret := self listMorph selectedRow ifNil: [ 0 ].
	"bindings stub"
	self bindings triggerEvent: #accessed with: (AccessDescriptorMessage new
			accessedKeyPath: #(selectionIndex);
			lastSender: self;
			accessedValue: ret).
	"return the index we have currently selected, or 0 if none"
	^ ret! !

!SelectableListMorph methodsFor: 'selection' stamp: 'topa 2/9/2010 18:39'!
selectionIndex: index
	"Called from outside to request setting a new selection."

	<bindable>
	| row selectionDesc selectionIndexDesc item |
	"bindings stub"
	(self listMorph selectedRow = index) ifTrue: [^self].
	item := (index = 0) ifTrue: [nil] ifFalse:[self getList at: index].
	selectionDesc := ChangeDescriptorMessage new
		operation: #selection; keyPathOfChange: #(); lastSender: self; changedValue: item.
	selectionIndexDesc := ChangeDescriptorMessage new
		operation: #selectionIndex; keyPathOfChange: #(); lastSender: self; changedValue: index.
	self bindings triggerEvent: #aboutToChange with: selectionDesc.
	self bindings triggerEvent: #aboutToChange with: selectionIndexDesc.

	"Called internally to select the index-th item."
	self unhighlightSelection.
	row := index ifNil: [ 0 ].
	row := row min: self getListSize.  "make sure we don't select past the end"
	self listMorph selectedRow: row.
	self highlightSelection.
	self scrollSelectionIntoView.
	
	self bindings triggerEvent: #hasChanged with: selectionIndexDesc.
	self bindings triggerEvent: #hasChanged with: selectionDesc.
! !

TestCase subclass: #BindingsTestCase
	instanceVariableNames: 'sut helper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

BindingsTestCase subclass: #BindingsCollectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsCollectionTest methodsFor: 'running' stamp: 'topa 1/11/2010 15:18'!
setUp

	self 
		sut: BindingsTestObjectWithCollections new;
		helper: BindingsTestObjectWithCollections new.! !

!BindingsCollectionTest methodsFor: 'tests' stamp: 'AK 2/6/2010 09:16'!
testArrayKeyPath

	| bindingsSize |
	bindingsSize := self sut bindings observedBindings size.
	
	self sut bind: #d to: self helper with: #(d selection a) as: #value.
	self should: [bindingsSize + 2 =  self sut bindings observedBindings size] 
		description: 'Bindings with key paths should be able to establish'.
	
	self sut unbind: #d from: self helper with: #(d selection a).
	self should: [bindingsSize =  self sut bindings observedBindings size] 
		description: 'Bindings with key paths should be able to remove'.! !

!BindingsCollectionTest methodsFor: 'tests' stamp: 'topa 1/11/2010 17:14'!
testCreationRemovalBind

	| sutDSize helperDSize |
	self sut bind: #d to: self helper with: #d.
	
	sutDSize := self sut d size.
	helperDSize := self helper d size.
	
	self sut d add: 1.
	self should: [(sutDSize + 1 = self sut d size) and: [helperDSize + 1 = self helper d size]]	
		description: 'Bound colections should be both one larger after addition'.

	self helper d remove: 1.
	self should: [(self sut d size = sutDSize) and: [ self helper d size = helperDSize]]	
		description: 'Bound colections should be both one smaller after removal'.

	self sut unbind: #d from: self helper with: #d.! !

!BindingsCollectionTest methodsFor: 'tests' stamp: 'topa 1/18/2010 16:52'!
testKeyPathSemantics

	| bindingsSize |
	bindingsSize := self sut bindings observedBindings size.
	
	self sut bind: #d to: self helper with: #(d selection a).
	self should: [bindingsSize + 1 =  self sut bindings observedBindings size] 
		description: 'Bindings with key paths should be able to establish'.
	
	self sut unbind: #d from: self helper with: #(d selection a).
	self should: [bindingsSize =  self sut bindings observedBindings size] 
		description: 'Bindings with key paths should be able to remove'.! !

BindingsTestCase subclass: #BindingsDescriptorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsDescriptorTest methodsFor: 'running' stamp: 'topa 12/29/2009 18:19'!
setUp

	super setUp.
	self
		sut: BindingsDescriptor new;
		helper: BindingsTestObject new.! !

!BindingsDescriptorTest methodsFor: 'testing' stamp: 'topa 2/8/2010 09:43'!
testDescriptorCreation

	| bindings |
	self helper bind: #a to: self helper with: #a.
	
	bindings := self helper bindings observedBindings at: (self bringToCanonicalForm: #a).
	bindings do: [:binding | 
		self should: [binding isBindingDescriptor]
				description: 'Binding descriptors should be created automatically upon binding creation'].
	
	self helper unbind: #a from: self helper with: #a.! !

!BindingsDescriptorTest methodsFor: 'testing' stamp: 'topa 1/18/2010 16:02'!
testDescriptorEquality

	| comparand |
	self sut 
		origin: self helper;
		localKeyPath: #a;
		target: self helper;
		keyPath: #a.
		
	comparand := BindingsDescriptor origin: self helper localKeyPath: #a target: self helper keyPath: #a.
	
	self should: [(self sut = comparand) and: [self sut hash = comparand hash]]
			description: 'Content-equal Bindings-Descriptors should be treated as equal';
		shouldnt: [self sut == comparand]
			description: 'Content-equal Bindings-Descriptors should not be treated as identical'.
	! !

BindingsTestCase subclass: #BindingsManagedCollectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsManagedCollectionTest methodsFor: 'running' stamp: 'topa 1/11/2010 17:17'!
setUp

	super setUp.
	self 
		sut: ManagedCollection new;
		helper: ManagedCollection new.
	! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:54'!
testAddToCollection
	| sutFirst sutSecond |

	self sut bind: #a to: self helper with: #a.

	sutFirst := BindingsTestObject new.
	sutFirst a: 10.
	sutSecond  := BindingsTestObject new.
	sutSecond a: 20.
	
	self sut add: sutFirst.
	self sut add: sutSecond.
	
	self 
		should: [self helper first a = 10];
		should: [self helper second b = 20].! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 18:20'!
testHasArrangedObjects

	self
		should: [self sut respondsTo: #arrangedObjects] description: 'A Managed Collection should provide all arrangedObjects';
		should: [self sut class exposedBindings includes: #arrangedObjects] description: 'The arranged Objects should be bindable'.
! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 18:04'!
testHasBindingsManager

	self
		should: [self sut respondsTo: #bindings] 
			description: 'A Managed Collection should have a Bindigns manager'.
		! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 18:20'!
testHasSelection

	self
		should: [self sut respondsTo: #selection] description: 'A Managed Collection should provide a selection';
		should: [self sut class exposedBindings includes: #selection] description: 'A selection should be bindable'.
		! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:54'!
testManagedCollection
	| sutFirst sutSecond |

	self sut: BindingsTestCollection new.
	self sut bind: #a to: self helper with: #a.

	sutFirst := BindingsTestObject new.
	sutFirst a: 10.
	sutSecond  := BindingsTestObject new.
	sutSecond a: 20.
	
	self sut add: sutFirst.
	self sut add: sutSecond.
	self sut selectObject: sutFirst.
	
	self should: [self helper selection a = 10].! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:54'!
testReadFromCollectionChange
	| sutFirst sutSecond |

	self sut bind: #a to: self helper with: #a.

	sutFirst := BindingsTestObject new.
	sutFirst a: 10.
	sutSecond  := BindingsTestObject new.
	sutSecond a: 20.
	
	self helper add: sutFirst.
	self helper add: sutSecond.
	
	self 
		should: [self sut first a = 10];
		should: [self sut second a = 20].! !

!BindingsManagedCollectionTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:54'!
testRemoveFromCollection
	| sutFirst sutSecond |

	self sut bind: #a to: self helper with: #a.

	sutFirst := BindingsTestObject new.
	sutFirst a: 10.
	sutSecond  := BindingsTestObject new.
	sutSecond a: 20.
	
	self sut add: sutFirst.
	self sut add: sutSecond.
	self sut remove: sutFirst.
	
	self 
		should: [self helper first a = 20];
		should: [self helper size = 1].! !

BindingsTestCase subclass: #BindingsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsTest methodsFor: 'running' stamp: 'topa 12/29/2009 18:19'!
setUp

	super setUp.
	self 
		sut: BindingsTestObject new;
		helper: BindingsTestObject new.! !

!BindingsTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:54'!
testChainedBinding
	
	| theValue secondHelper |
	theValue := 42.
	secondHelper := BindingsTestObject new.
	self sut bind: #a to: self helper with: #a.
	self helper bind: #a to: secondHelper with: #a.
	
	secondHelper a: theValue.
	self should: [(self helper a = theValue) and: [self sut a = theValue]]
		description: 'Bound Morphs should have a value set by the binding target, even chained'.
	
	self sut unbind: #a from: self helper with: #a.
! !

!BindingsTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:55'!
testCreateRemoveBindings
	
	| numOfBindings numOfBindingsTwoWay |
	numOfBindings := self sut bindings observedBindings size.
	numOfBindingsTwoWay := self helper bindings observedBindings size.
	
	
	self sut bind: #a to: self helper with: #a.	
	self should: [self sut bindings observedBindings size = (numOfBindings + 1)]
		description: 'Binding Morphs should increase the size of the observed bindings'.
	self should: [self sut bindings target = self sut]
		description: 'Binding Manager should know its target.'.
	self should: [self helper bindings observedBindings size = (numOfBindingsTwoWay + 1)]
		description: 'Two Way Binding Morphs should increase the size of the observed bindings at the target, too'.
		

	self sut unbind: #a from: self helper with: #a.
	self should: [self sut bindings observedBindings size = numOfBindings]
		description: ' Unbinding Morphs should decrease the size of the observed bindings'.	
	self should: [self helper bindings observedBindings size = numOfBindingsTwoWay]
		description: ' Unbinding Morphs should decrease the size of the observed bindings at the target in a two way binding'.	
! !

!BindingsTest methodsFor: 'testing' stamp: 'topa 12/29/2009 18:21'!
testExposedBindings

	self 
		should: [self sut class exposedBindings includes: #a];
		should: [self sut class exposedBindings includes: #a:];
		should: [self sut class exposedBindings size = 2];
		shouldnt: [self sut class exposedBindings includes: #b];
		shouldnt: [self sut class exposedBindings includes: #b:].
	! !

!BindingsTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:55'!
testReadBinding
	
	| theValue |
	theValue := 42.
	self sut bind: #a to: self helper with: #a.
	self helper a: theValue.	
	self should: [self sut a = theValue]
		description: 'Bound Morphs should have a value set by the binding target'.
	
	self sut unbind: #a from: self helper with: #a.
! !

!BindingsTest methodsFor: 'testing' stamp: 'topa 1/11/2010 17:55'!
testReadWriteBinding
	
	| theValue |
	theValue := 42.
	self sut bind: #a to: self helper with: #a.
	self helper a: theValue.	
	self should: [self sut a = theValue]
		description: 'Bound Morphs should have a value set by the binding target'.
	self should: [self helper a = theValue]
		description: 'Bound Morphs should should not change their value unconditionally'.
	theValue := 16.
	self sut a: theValue.
	self should: [self helper a = theValue]
		description: 'Bound Targets should have a value set by the binding Morphs, too'.
	
	self sut unbind: #a from: self helper with: #a.! !

!BindingsTestCase class methodsFor: 'testing' stamp: 'topa 12/29/2009 18:24'!
isAbstract

	^self name = #BindingsTestCase! !

!BindingsTestCase methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:15'!
helper

	^ helper! !

!BindingsTestCase methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:15'!
helper: aTestHelperObject

	helper := aTestHelperObject.! !

!BindingsTestCase methodsFor: 'running' stamp: 'topa 12/29/2009 18:16'!
setUp

	super setUp.
	self
		sut: nil;
		helper: nil.! !

!BindingsTestCase methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:15'!
sut

	^ sut! !

!BindingsTestCase methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:14'!
sut: aSystemUnderTest

	sut := aSystemUnderTest.! !

!BindingsTestCase methodsFor: 'running' stamp: 'topa 12/29/2009 18:13'!
tearDown

	self
		sut: nil;
		helper: nil.
	super tearDown! !

Object subclass: #AccessDescriptorMessage
	instanceVariableNames: 'accessedKeyPath lastSender accessedValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Base'!

!AccessDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 09:55'!
accessedKeyPath

	^ accessedKeyPath! !

!AccessDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 09:55'!
accessedKeyPath: keyPath

	accessedKeyPath := keyPath.! !

!AccessDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 09:55'!
accessedValue

	^ accessedValue! !

!AccessDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 09:55'!
accessedValue: aValue

	accessedValue := aValue.! !

!AccessDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 09:55'!
lastSender

	^ lastSender! !

!AccessDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 09:55'!
lastSender: sender

	lastSender := sender.! !

Object subclass: #BindingsDescriptor
	instanceVariableNames: 'origin target localKeyPath keyPath bindingType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Base'!

!BindingsDescriptor class methodsFor: 'instance creation' stamp: 'AK 2/5/2010 23:19'!
as: bindingType
origin: aMorph
localKeyPath: aSymbol
target: anotherMorph
keyPath: aKeyPath

	^ self new
		bindingType: bindingType;
		origin: aMorph;
		localKeyPath: aSymbol;
		target: anotherMorph;
		keyPath: aKeyPath! !

!BindingsDescriptor class methodsFor: 'instance creation' stamp: 'topa 1/18/2010 16:01'!
origin: aMorph
localKeyPath: aSymbol
target: anotherMorph
keyPath: aKeyPath

	^ self new
		origin: aMorph;
		localKeyPath: aSymbol;
		target: anotherMorph;
		keyPath: aKeyPath! !

!BindingsDescriptor methodsFor: 'comparing' stamp: 'topa 2/9/2010 09:58'!
= aBindingsDescriptor
	"Bindings Descriptors are identical iff they have the same inner values"

	^ self == aBindingsDescriptor
		ifTrue: [true]
		ifFalse:[
			(self origin     = aBindingsDescriptor origin) and: [
			(self localKeyPath = aBindingsDescriptor localKeyPath) and: [
			(self target    = aBindingsDescriptor target) and: [
			(self keyPath  = aBindingsDescriptor keyPath)]]]]
			! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 2/9/2010 09:58'!
bindingType

	^ bindingType! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 2/9/2010 09:58'!
bindingType: type

	bindingType := type.! !

!BindingsDescriptor methodsFor: 'comparing' stamp: 'topa 1/18/2010 15:59'!
complement

	^ self class
		origin: self target
		localKeyPath: self keyPath
		target: self origin
		keyPath: self localKeyPath
! !

!BindingsDescriptor methodsFor: 'comparing' stamp: 'topa 1/18/2010 15:59'!
hash

	^ (((self origin hash 
		bitXor: self localKeyPath hash) 
			bitXor: self target hash)
				bitXor: self keyPath hash)! !

!BindingsDescriptor methodsFor: 'testing' stamp: 'topa 12/29/2009 17:01'!
isBindingDescriptor

	^ true! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:35'!
keyPath

	^ keyPath! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:35'!
keyPath: anObject

	keyPath := anObject.! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 1/18/2010 15:56'!
localKeyPath

	^ localKeyPath! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 1/18/2010 15:58'!
localKeyPath: anObject

	localKeyPath := anObject.! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:36'!
origin

	^ origin! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:34'!
origin: anObject

	origin := anObject.! !

!BindingsDescriptor methodsFor: 'printing' stamp: 'topa 1/18/2010 15:59'!
printOn: aStream

	super printOn: aStream.
	aStream
		nextPut: $(;
		print: self origin; nextPutAll: ''; print: self localKeyPath;
		nextPutAll: ' => ';
		print: self target; nextPutAll: ''; print: self keyPath;
		nextPut: $)! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:36'!
target

	^ target! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:35'!
target: anObject

	target := anObject.! !

Object subclass: #BindingsManager
	instanceVariableNames: 'observedBindings innerBindings target propertyName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Base'!

!BindingsManager methodsFor: 'reacting' stamp: 'topa 2/9/2010 09:58'!
aboutToChange: aBindingAssoc
	"aboutToChange: is just an option to react
	if s.b. wants to change, a.t.m. nobody reacts"! !

!BindingsManager methodsFor: 'reacting' stamp: 'topa 2/9/2010 09:58'!
accessed: anAccessDescriptorMessage
	"accessed is just an option to react, if somebody
	has accessed a variable and a.t.m. is not implemented"! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/8/2010 09:56'!
allInnerBindings
	"Collect all inner bindings"

	^ OrderedCollection streamContents: [ :stream |
		self innerBindings valuesDo: [:bindings |
			stream nextPutAll: bindings]]
! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/8/2010 09:56'!
allObservedBindings
	"Collect all observed bindings"

	^ OrderedCollection streamContents: [ :stream |
		self observedBindings valuesDo: [:bindings |
			stream nextPutAll: bindings]]! !

!BindingsManager methodsFor: 'termination' stamp: 'topa 2/9/2010 09:57'!
alreadyContainsKey: localKeyPath
	" ^ <Boolean> "

	^ (self observedBindings 
		at: localKeyPath 
		ifAbsent: [OrderedCollection new])
			isEmpty not! !

!BindingsManager methodsFor: 'termination' stamp: 'AK 2/5/2010 18:23'!
alreadyContainsKey: localKeyPath regarding: aMorph with: aKeyPath

	| foundDescriptors wantedDescriptor |
	wantedDescriptor := (BindingsDescriptor origin: (self target) localKeyPath: localKeyPath target: aMorph keyPath: aKeyPath).

	foundDescriptors := self observedBindings at: localKeyPath ifAbsent: [OrderedCollection new].
	
	foundDescriptors do: [ :foundDescriptor |
		((foundDescriptor target == wantedDescriptor target) and:
		[foundDescriptor keyPath == wantedDescriptor keyPath])
			ifTrue: [
				^true
				]].
	^false.! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/9/2010 09:59'!
bindingDescriptorForChangeDescriptor: aChangeDescriptor
	"For a BindingsAssociation that arrives from the other side,
	the corresponding (already set up) BindingsDescriptor is returned"

	|  target keyPath |
	target := aChangeDescriptor lastSender.
	keyPath := aChangeDescriptor keyPathOfChange.

	^ self allObservedBindings 
		detect: [:bd| 
			(bd target = target) and: [
			(bd keyPath = keyPath) and: [
			bd origin = self target]]]
		ifNone: [nil]! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:00'!
bindingDescriptorsForInnerChangeDescriptor: aChangeDescriptor
	"For a BindingsAssociation that arrives from the other side,
	the corresponding (already set up) BindingsDescriptor is returned"
	
	|  target keyPath foundDescriptors |
	target := aChangeDescriptor lastSender.
	keyPath := aChangeDescriptor keyPathOfChange.
	foundDescriptors := OrderedCollection new.

	self allInnerBindings 
		do: [:bd| 
			((bd target = target) and: [
			(bd keyPath = keyPath) and: [
			bd origin = self target]])
				ifTrue: [	foundDescriptors add: bd]].
	^foundDescriptors.! !

!BindingsManager methodsFor: 'termination' stamp: 'topa 1/18/2010 16:02'!
forgetBinding: aBindingsDescriptor

	(self observedKey: aBindingsDescriptor localKeyPath) 
		remove: aBindingsDescriptor ifAbsent: ["fallthough"].
	self removeIfObsoleteKey: aBindingsDescriptor localKeyPath
! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:02'!
hasChanged: aChangeDescriptorMessage
	"if a value has changed on the other side, then this method
	is called with a ChangeDescriptor Message
	The BindingsDescriptor is searched and the Morph setter called."

	| descriptor innerDescriptors |
	descriptor := self bindingDescriptorForChangeDescriptor: aChangeDescriptorMessage.
	innerDescriptors := self bindingDescriptorsForInnerChangeDescriptor: aChangeDescriptorMessage.

	"is incoming message from other side?"
	descriptor ifNotNil: [self otherSideHasChanged: descriptor].
		
	"If incoming message from inner bindings, then:
	Send ChangeDescriptorMessage to next receiver"
	innerDescriptors do: [:bindingDescriptor |
		"Change ChangeDescriptorMessage's lastSender and keypath"
		aChangeDescriptorMessage 
			lastSender: self target;
			keyPathOfChange: bindingDescriptor localKeyPath.
		self triggerEvent: #hasChanged with: aChangeDescriptorMessage].! !

!BindingsManager methodsFor: 'initialize-release' stamp: 'topa 2/9/2010 10:11'!
initialize

	self 
		observedBindings: Dictionary new;
		innerBindings: Dictionary new.! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:03'!
innerBindings

	^ innerBindings! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:03'!
innerBindings: aDict

	innerBindings := aDict.! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:03'!
innerKey: aKey
	"Fetches one observed binding"

	^ self innerBindings at: aKey ifAbsentPut: [Set new]! !

!BindingsManager methodsFor: 'establishing' stamp: 'topa 2/9/2010 10:11'!
installEventHandlers: aBindingsDescriptor
	"makes the BindingsManager on the other side react
	on events with accessed:/aboutToChange:/hasChanged:"

	#( accessed aboutToChange hasChanged ) do: [:eventSymbol |
		aBindingsDescriptor target bindings 
			when: eventSymbol
			send: eventSymbol asMutator
			to: aBindingsDescriptor origin bindings].! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:04'!
observedBindings

	^ observedBindings! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:04'!
observedBindings: aCollection

	observedBindings := aCollection.! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:04'!
observedKey: aKey
	"Fetches a set of observed bindings for that key"

	^ self observedBindings at: aKey ifAbsentPut: [Set new]! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:10'!
otherSideHasChanged: descriptor
	"if a value has changed on the other side, then this method
	is called with the Descriptor."

	(descriptor bindingType == #collection) ifTrue: [
		| ownvalues othervalues |
		"fetch values from the other side"
		othervalues := descriptor target getValuesFromKeypath: descriptor keyPath.
		"get the collection from this side"
		ownvalues := self target getCollectionsFromKeypath: descriptor localKeyPath.
		
		self flag: #todo. 
			"look if the target collection has any set up innerbinding.
			then it must have string values."		
			"else it must have complex objects as values."

		"remove all values, iterate over collection and add values"
		ownvalues do: [:collection |
			collection removeAll.
			othervalues do: [ :value | collection add: value]]].
				
	(descriptor bindingType == #value) ifTrue: [
		| ownmessageName ownvalues othermessageName othervalues |
		"fetch the last symbol"
		ownmessageName := descriptor localKeyPath last asMutator.
		"get the parent value object with the remaining symbols"
		ownvalues := descriptor origin getValuesFromKeypath: descriptor localKeyPath allButLast.
		"get the other side's message name"
		othermessageName := descriptor keyPath last.
		"get the other side's value object with the other side's descriptor"
		othervalues := descriptor target getValuesFromKeypath: descriptor keyPath allButLast.
		"set the first found value with first value from the other side"
		ownvalues first perform: ownmessageName with: (othervalues first perform: othermessageName)].! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:12'!
propertyName

	^ propertyName! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:12'!
propertyName: name

	propertyName := propertyName.! !

!BindingsManager methodsFor: 'establishing' stamp: 'topa 2/9/2010 10:18'!
recordBinding: aBindingsDescriptor
	
	(self observedKey: aBindingsDescriptor localKeyPath) 
		add: aBindingsDescriptor.! !

!BindingsManager methodsFor: 'establishing' stamp: 'topa 2/9/2010 10:18'!
recordInnerBinding: aBindingsDescriptor

	(self innerKey: aBindingsDescriptor localKeyPath) 
		add: aBindingsDescriptor.! !

!BindingsManager methodsFor: 'establishing' stamp: 'topa 2/9/2010 10:18'!
recordObservedBinding: aBindingsDescriptor

	(self observedKey: aBindingsDescriptor localKeyPath) 
		add: aBindingsDescriptor.! !

!BindingsManager methodsFor: 'termination' stamp: 'topa 1/11/2010 15:14'!
removeEventHandlers: aBindingsDescriptor

	#( accessed aboutToChange hasChanged ) do: [:eventSymbol |
			aBindingsDescriptor target bindings 
				removeActionsWithReceiver: (aBindingsDescriptor origin bindings) 
				forEvent: eventSymbol].
! !

!BindingsManager methodsFor: 'termination' stamp: 'topa 12/27/2009 18:10'!
removeIfObsoleteKey: aKey

	((self observedBindings includesKey: aKey) and: [
		(self observedBindings at: aKey) isEmpty])
			ifTrue: [	self observedBindings removeKey: aKey]! !

!BindingsManager methodsFor: 'establishing' stamp: 'topa 2/9/2010 10:19'!
startObservingKey: localKeyPath as: type notifying: aMorph with: aKeyPath
	"builds a Bindings Descriptor for new Binding, adds it and installs it"

	| descriptor |
	descriptor := (BindingsDescriptor as: type origin: (self target) localKeyPath: localKeyPath target: aMorph keyPath: aKeyPath).
	self
		recordObservedBinding: descriptor;
		installEventHandlers: descriptor.
! !

!BindingsManager methodsFor: 'termination' stamp: 'topa 2/9/2010 10:22'!
stopObservingKey: localKeyPath regarding: aMorph with: aKeyPath

	| foundDescriptors wantedDescriptor |
	
	wantedDescriptor := (BindingsDescriptor origin: (self target) localKeyPath: localKeyPath target: aMorph keyPath: aKeyPath).
	foundDescriptors := self observedBindings at: localKeyPath ifAbsent: [OrderedCollection new].
	
	foundDescriptors do: [:foundDescriptor |
		((foundDescriptor target = wantedDescriptor target) and:
			[foundDescriptor keyPath = wantedDescriptor keyPath])
			ifTrue: [
				self 
					forgetBinding: foundDescriptor;
					removeEventHandlers: foundDescriptor]].! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 12/29/2009 12:45'!
target

	^ target! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:22'!
target: aMorph

	target := aMorph.! !

!BindingsManager methodsFor: 'establishing' stamp: 'topa 2/9/2010 10:23'!
watchInnerKey: localKeyPath as: type notifying: aMorph with: aKeyPath
	"builds a Bindings Descriptor for new Binding, adds it and installs it"

	| descriptor |
	descriptor := (BindingsDescriptor as: type origin: (self target) localKeyPath: localKeyPath target: aMorph keyPath: aKeyPath).
	self
		recordInnerBinding: descriptor;
		installEventHandlers: descriptor.
! !

Object subclass: #BindingsManagerHub
	instanceVariableNames: 'bindingsManagers'
	classVariableNames: 'Instance'
	poolDictionaries: ''
	category: 'Bindings-Base'!

!BindingsManagerHub class methodsFor: 'accessing' stamp: 'topa 1/26/2010 10:14'!
instance

	^ Instance ifNil: [Instance := self basicNew initialize]! !

!BindingsManagerHub class methodsFor: 'instance creation' stamp: 'topa 1/26/2010 10:13'!
new

	^ self shouldNotImplement! !

!BindingsManagerHub methodsFor: 'initialize-release' stamp: 'topa 1/26/2010 10:19'!
bindingsManagerFor: anObject

	^ self bindingsManagers 
		at: anObject 
		ifAbsentPut: [self class bindingsManagerClass new target: anObject]
! !

!BindingsManagerHub methodsFor: 'accessing' stamp: 'topa 1/26/2010 10:17'!
bindingsManagers

	^ bindingsManagers! !

!BindingsManagerHub methodsFor: 'accessing' stamp: 'topa 1/26/2010 10:16'!
bindingsManagers: aDictionary

	bindingsManagers := aDictionary.! !

!BindingsManagerHub methodsFor: 'initialize-release' stamp: 'topa 1/26/2010 10:41'!
initialize

	super initialize.
	self bindingsManagers: WeakIdentityKeyDictionary new.! !

Object subclass: #BindingsTestObject
	instanceVariableNames: 'a b'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsTestObject methodsFor: 'accessing' stamp: 'topa 12/27/2009 13:15'!
a
	
	<bindable>
	^ self valueOfVariableNamed: #a! !

!BindingsTestObject methodsFor: 'accessing' stamp: 'topa 12/27/2009 13:15'!
a: anObject
	
	<bindable>
	self setVariableNamed: #a toValue: anObject! !

!BindingsTestObject methodsFor: 'accessing' stamp: 'topa 12/27/2009 13:16'!
b

	^ b! !

!BindingsTestObject methodsFor: 'accessing' stamp: 'topa 12/27/2009 13:16'!
b: anObject

	b := anObject! !

BindingsTestObject subclass: #BindingsTestObjectWithCollections
	instanceVariableNames: 'd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsTestObjectWithCollections methodsFor: 'accessing' stamp: 'topa 1/11/2010 17:56'!
d

	<bindableAsCollection>
	^ self valueOfVariableNamed: #d! !

!BindingsTestObjectWithCollections methodsFor: 'accessing' stamp: 'topa 1/11/2010 17:56'!
d: anObject

	<bindableAsCollection>
	self setVariableNamed: #d toValue: anObject.! !

!BindingsTestObjectWithCollections methodsFor: 'as yet unclassified' stamp: 'topa 1/11/2010 15:15'!
initialize

	self d: OrderedCollection new.! !

Object subclass: #ChangeDescriptorMessage
	instanceVariableNames: 'lastSender keyPathOfChange operation changedValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Base'!

!ChangeDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:25'!
changedValue

	^ changedValue! !

!ChangeDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:25'!
changedValue: value

	changedValue := value.! !

!ChangeDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:25'!
keyPathOfChange

	^ keyPathOfChange! !

!ChangeDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:25'!
keyPathOfChange: keyPath

	keyPathOfChange := keyPath.! !

!ChangeDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:25'!
lastSender

	^ lastSender! !

!ChangeDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:25'!
lastSender: sender

	lastSender := sender.! !

!ChangeDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:25'!
operation

	^ operation! !

!ChangeDescriptorMessage methodsFor: 'accessing' stamp: 'topa 2/9/2010 10:25'!
operation: anOperation

	operation := anOperation.! !

!Object class methodsFor: '*bindings' stamp: 'topa 1/11/2010 18:17'!
bindingsManagerClass

	^ BindingsManager! !

!Object class methodsFor: '*bindings' stamp: 'AK 1/31/2010 11:57'!
bindingsManagerHub
"Fetches the global Manager Hub that has all existing BindingsManagers"

	^ BindingsManagerHub instance! !

!Object class methodsFor: '*bindings' stamp: 'AK 2/5/2010 19:39'!
exposedBindings
"Fetches all own exposed Bindings (Methods that have the #bindable pragma)"
	
	^ Array streamContents: [ :stream |
		stream nextPutAll: self exposedValueBindings.
		stream nextPutAll: self exposedCollectionBindings]
! !

!Object class methodsFor: '*bindings' stamp: 'AK 1/31/2010 11:57'!
exposedCollectionBindings
"Fetches all own exposed Collection Bindings (i.e. Methods that have the #bindableAsCollection pragma)"

	^ Array streamContents: [ :stream |
		(Pragma allNamed: #bindableAsCollection in: self) do: [:pragma |
			stream nextPut: pragma selector]]! !

!Object class methodsFor: '*bindings' stamp: 'AK 2/5/2010 19:39'!
exposedValueBindings
"Fetches all own exposed Collection Bindings (i.e. Methods that have the #bindableAsCollection pragma)"

	^ Array streamContents: [ :stream |
		(Pragma allNamed: #bindable in: self) do: [:pragma |
			stream nextPut: pragma selector]]
	! !

!Object class methodsFor: '*bindings' stamp: 'AK 1/31/2010 12:00'!
generateBindingsAccessors: selectors
"For the given selectors, this code will generate
own bindings accessors (i.e. getters and setters)"

	| getterTempl setterTempl methodCategory |
	getterTempl := '<MSG>\	"Generated automatically!!"\\	<bindable>\	^ self valueOfVariableNamed: #<MSG>' withCRs.
	setterTempl := '<MSG>: anObject\	"Generated automatically!!"\\	<bindable>\	self setVariableNamed: #<MSG> toValue: anObject.' withCRs.
	methodCategory := #'accessing-generated'. 

	{ getterTempl. setterTempl } do: [:template|
		selectors do: [:selector| | code |
			code := template copyReplaceTokens: '<MSG>' with: selector.
			self compile: code classified: methodCategory]]! !

!Object class methodsFor: '*bindings' stamp: 'AK 1/31/2010 12:01'!
managedCollectionClass
"Returns a class ManagedCollection, that exists for
wrapping Bindable Collections into s.th. compatible with bindings"

	^ ManagedCollection! !

!Object methodsFor: '*bindings' stamp: 'AK 2/6/2010 09:18'!
bind: bindable to: anObject with: aKeyPath

	self bind: bindable to: anObject with: aKeyPath as: #value.! !

!Object methodsFor: '*bindings' stamp: 'AK 2/6/2010 11:25'!
bind: bindable to: anObject with: aKeyPath as: bindingtype
"Binds the bindable of the current binder/bindee to a model described by model key path.
Keypath has the form: #(collectionname arrangedObjects/selection valuename).
Bindingtype can either be #value or #collection."
	
	| canonicalFormOfBindable canonicalFormOfKeypath |
	canonicalFormOfBindable := self bringToCanonicalForm: bindable.
	canonicalFormOfKeypath := self bringToCanonicalForm: aKeyPath.
	
	"Check if name of exposed variable is inside this binder/bindee"
	(self exposedBindings includes: canonicalFormOfBindable first) ifTrue: [

	"Check if the other side has already bound to the key"
	(self bindings alreadyContainsKey: canonicalFormOfBindable regarding: anObject with: canonicalFormOfKeypath) ifFalse: [

		"Check if more difficult binding is set up"
		((self isBindableCollection: canonicalFormOfBindable))
			ifTrue: [	
					self innerbind: canonicalFormOfBindable
					withProperty: nil
					as: bindingtype]
			ifFalse: [
				"handles a simple binding"
				
				].

		"Remember morph on the other side (with it's keypath)"
		(self bindings startObservingKey: canonicalFormOfBindable as: bindingtype notifying: anObject with: canonicalFormOfKeypath).
		"Init notification mechanism at other morph (object notifies morph notifies me)"
		anObject bind: canonicalFormOfKeypath to: self with: canonicalFormOfBindable as: bindingtype]].! !

!Object methodsFor: '*bindings' stamp: 'AK 1/31/2010 11:25'!
bindings
"returns own bindings manager (from the global hub)"

	^ self class bindingsManagerHub
		bindingsManagerFor: self! !

!Object methodsFor: '*bindings' stamp: 'topa 2/8/2010 09:42'!
bringToCanonicalForm: aKeyPath
	"Keypath is in its canonical form if it is in an array"

	^ aKeyPath isSymbol
		ifTrue: [Array with: aKeyPath]
		ifFalse: [aKeyPath]
! !

!Object methodsFor: '*bindings' stamp: 'AK 1/31/2010 11:29'!
ensureManagedCollection: bindable
"if the bindable on the other side is a managed collection
otherwise the collection on the other side is wrapped"

	| mc |
	mc := self instVarNamed: bindable.
			
	(mc isNil or: [mc isManagedCollection not])
		ifTrue: [
			mc := (self class managedCollectionClass wrapping: mc).
			self instVarNamed: bindable put: mc].
	^ mc! !

!Object methodsFor: '*bindings' stamp: 'topa 1/26/2010 10:24'!
exposedBindings

	^ self class exposedBindings
! !

!Object methodsFor: '*bindings' stamp: 'topa 2/9/2010 18:53'!
getCollectionsFromKeypath: aKeyPath
"Implementation of the Keypath Pattern language for values.
Searches for all results from a given keypath on this object.
Returns all ManagedCollections that wrap the referenced objects/colls."
	
	| objectContext nextObjectContext remainingKeyPath currentKey objectBehindKey keyIterator lastManagedCollections lastMCLevel |
	
	self flag: #NeedsRefactoring.
	keyIterator := 0.
	lastMCLevel := 0.
	aKeyPath ifNil: [^nil].
	
	"should not be nil or something else than collection&symbol"
	remainingKeyPath := aKeyPath isArray
		ifTrue: [aKeyPath]
		ifFalse: 	[aKeyPath isSymbol
			ifTrue: [Array with: aKeyPath]
			ifFalse:	[^nil]].
	
	"initializes the object context with self"
	objectContext := OrderedCollection with: self.
	nextObjectContext := OrderedCollection new.
	
	[remainingKeyPath isEmpty] whileFalse: [
		keyIterator := keyIterator + 1.
		currentKey := remainingKeyPath first.
		remainingKeyPath := remainingKeyPath allButFirst.
		
		"(A) arrangedObject"
		currentKey == #arrangedObjects ifTrue: [self error: 'arrangedObjects can''t stand alone'. ^nil].
		"(B) selection"
		currentKey == #selection ifTrue: [self error: 'selection can''t stand alone'. ^nil].
		
		objectContext do: [ :contextObject |
			"TODO: check if it exists"
			objectBehindKey := (contextObject instVarNamedOrNil: currentKey).
			
			"(C) collection"
			objectBehindKey isCollection
				ifTrue: [
					((objectBehindKey isManagedCollection) and: 
						[keyIterator > lastMCLevel]) 
						ifTrue: [	"fetch all collections"
								keyIterator := lastMCLevel.
								lastManagedCollections := OrderedCollection new].
					
					"(C.1) collection & end"
					remainingKeyPath isEmpty
						ifTrue: [
							objectBehindKey isManagedCollection ifTrue: ["fetch all collections" lastManagedCollections add: objectBehindKey]
						ifFalse: [
							"COLLECTION IS NOT MANAGEDCOLLECTION"
							"nextObjectContext add: objectBehindKey." ]]
				ifFalse: [
					currentKey := remainingKeyPath first.
					remainingKeyPath := remainingKeyPath allButFirst.
					
					"(C.2) collection & arrangedObject"
					currentKey == #arrangedObjects
					ifTrue: [
							"fetch all inner objects in all collections"
							objectBehindKey actualCollection do: [ :innerobject |
								innerobject ifNil: [
									nextObjectContext add: innerobject.
									lastManagedCollections add: innerobject.
									]
							].
						]
					ifFalse: [
							"(C.3) collection & selection"
							(currentKey == #selection)
							ifTrue: [
								"fetch the selection of all collections"
								objectBehindKey selection ifNil: [
									nextObjectContext add: (objectBehindKey selection)].
							]
							"(C.4) error, after collection the keypath must contain a keyword"
							ifFalse: [	
								"objectBehindKey do: [ :innerobject |
									innerobject ifNotNil: [
										nextObjectContext add: innerobject.
										]
									].
								remainingKeyPath := (Array with: currentKey) , 
											remainingKeyPath."
								"error, wrong variables referenced, wrong keypath, ignore"
								^nil
								].
						].
					]
			]
		ifFalse: [
			"(D) value"
			(objectBehindKey isNil)
			ifFalse: [
				(remainingKeyPath isEmpty)
					ifTrue: [
						"do not add the value" ]
					ifFalse: [
						"just a value in the keypath (because it is a step-through procedure), so add it"
						nextObjectContext add: objectBehindKey.

						 ].
				]
			"(E) does not exist"
			ifTrue: [
					"error, variable is not set, do nothing and ignore this value"
				].
			].
		].
		objectContext := nextObjectContext.
		nextObjectContext := OrderedCollection new.
	].

	^lastManagedCollections.! !

!Object methodsFor: '*bindings' stamp: 'topa 2/8/2010 09:48'!
getValuesFromKeypath: aKeyPath
"Implementation of the Keypath Pattern language for values.
Searches for all results from a given keypath on this object.
If it references to a collection, the collection is handled as a value."
	
	| objectContext nextObjectContext remainingKeyPath currentKey objectBehindKey |

	self flag: #NeedsRefactoring.	
	aKeyPath ifNil: [^nil].
	
	"should not be nil or something else than collection&symbol"
	(aKeyPath isArray)
	ifFalse: 
		[(aKeyPath isSymbol)
			ifTrue:
				[remainingKeyPath := Array with: aKeyPath]
			ifFalse:
				[^nil]]
	ifTrue:
		[remainingKeyPath := aKeyPath].
	
	"initializes the object context with self"
	objectContext := OrderedCollection with: self.
	nextObjectContext := OrderedCollection new.
	
	[remainingKeyPath isEmpty] whileFalse: [
		currentKey := remainingKeyPath first.
		remainingKeyPath := remainingKeyPath allButFirst.
		
		"(A) arrangedObject"
		(currentKey asString == #arrangedObject) ifTrue: [
				"error, arrangedObjects can't stand alone"
				^nil].
			
		"(B) selection"
		(currentKey asString == #selection) ifTrue: [
			"error, selection can't stand alone"
				^nil].
		
		objectContext do: [ :contextObject |
			"TODO: check if it exists"
			objectBehindKey := (contextObject instVarNamedOrNil: currentKey).
			
		"(C) collection"
		(objectBehindKey isCollection)
		ifTrue: [

			"(C.1) collection & end"
			(remainingKeyPath isEmpty)
				ifTrue: [
					(objectBehindKey isManagedCollection) ifTrue: [
						"fetch all collections"
						nextObjectContext add: objectBehindKey actualCollection.]
					ifFalse: [
						"fetch all collections"
						nextObjectContext add: objectBehindKey. ]
				]
				ifFalse: [
					currentKey := remainingKeyPath first.
					remainingKeyPath := remainingKeyPath allButFirst.
					
					"(C.2) collection & arrangedObject"
					(currentKey == #arrangedObjects)
					ifTrue: [
							"fetch all inner objects in all collections"
							objectBehindKey actualCollection do: [ :innerobject |
								innerobject ifNotNil: [
									nextObjectContext add: innerobject.
									]
							].
						]
					ifFalse: [
							"(C.3) collection & selection"
							(currentKey == #selection)
							ifTrue: [
								"fetch the selection of all collections"
								objectBehindKey selection ifNil: [
									nextObjectContext add: (objectBehindKey selection)].
							]
							"(C.4) error, after collection the keypath must contain a keyword"
							ifFalse: [	
								"objectBehindKey do: [ :innerobject |
									innerobject ifNotNil: [
										nextObjectContext add: innerobject.
										]
									].
								remainingKeyPath := (Array with: currentKey) , 
											remainingKeyPath."
								"error, wrong variables referenced, wrong keypath, ignore"
								^nil
								].
						].
					]
			]
		ifFalse: [
			"(D) value"
			(objectBehindKey isNil)
			ifFalse: [
				(remainingKeyPath isEmpty)
					ifTrue: [
						"do not add the value" ]
					ifFalse: [
						"just a value in the keypath (because it is a step-through procedure), so add it"
						nextObjectContext add: objectBehindKey.

						 ].
				]
			"(E) does not exist"
			ifTrue: [
					"error, variable is not set, do nothing and ignore this value"
				].
			].
		].
		objectContext := nextObjectContext.
		nextObjectContext := OrderedCollection new.
	].

	^objectContext.! !

!Object methodsFor: '*bindings' stamp: 'AK 2/6/2010 12:40'!
innerbind: keypath withProperty: property as: bindingtype
"Binds the inner bindables like collections to the parent recursively by stepping and cutting key path.
Keypath has the form: #([collectionname arrangedObjects/selection]* valuename) or #(collectionname) or #(valuename*)."
	
	| managedCollection |
	
	"Check if the key is already observed"
	(self bindings alreadyContainsKey: keypath) ifFalse: [
	
	"Check if name of exposed variable is inside this binder/bindee"
	(self class exposedBindings includes: keypath first)
		ifTrue: [
				
		"Check for simple case #(valuename)"
		((keypath size == 1) and: [self class exposedValueBindings includes: keypath first])
			ifTrue: [
					(bindingtype == #value) ifTrue: [ "The value is already observed"].
					(bindingtype == #collection) ifTrue: [ "observe collection values"
						"at the moment only add: and remove: supported"
						"and the value key is already recognized for this object"
						]
				].
	
		"Check for ordinary case #(collectionname)"
		((keypath size = 1) and: [self class exposedCollectionBindings includes: keypath first])
			ifTrue: [
					(bindingtype == #value) ifTrue: ["collection value is already observed"].
					(bindingtype == #collection) ifTrue: [ "Wrap into Managed Collection and bind collection to itself"
						managedCollection := self ensureManagedCollection: keypath first.
						(self bindings)
							watchInnerKey: keypath
							as: bindingtype
							notifying: managedCollection
							with: keypath allButFirst.
						].
				].
	
		"Check for complex case #(valuename ...)"
		((keypath size > 1) and: [self class exposedValueBindings includes: keypath first])
			ifTrue: [
				"Bind the value to us and recursively move on"
					(self bindings)
						watchInnerKey: keypath
						as: bindingtype
						notifying: (self instVarNamed: keypath first)
						with: keypath allButFirst.
					(self instVarNamed: keypath first)
						innerbind: (keypath allButFirst)
						withProperty: (keypath first)
						as: bindingtype.
				].
	
		"Check for complex case #(collectionname arrangedObjects ...)"
		((keypath size > 1) and: [self class exposedCollectionBindings includes: keypath first] and: [keypath second == #arrangedObjects])
			ifTrue: [
				(bindingtype == #value) ifTrue: ["This must be a mistake, returns a collection"Error new].
				(bindingtype == #collection) ifTrue: [
					"wraps the collection on this side into ManagedCollection, 
					bind collection on itself and hand over to arranged objects"
					managedCollection := self ensureManagedCollection: keypath first.
					(self bindings)
						watchInnerKey: keypath
						as: bindingtype
						notifying: managedCollection
						with: keypath allButFirst.
					managedCollection
						innerbind: (keypath allButFirst)
						withProperty: (keypath first) as: bindingtype.
					].	
				].

		"Check for complex case #(collectionname selection ...)"
		((keypath size > 1) and: [self class exposedCollectionBindings includes: keypath first] and: [keypath second == #selection])
				ifTrue: [
					"wraps the collection on this side into ManagedCollection, 
					bind collection on itself and hand over to selected objects"
					managedCollection := self ensureManagedCollection: keypath first.
					(self bindings)
						watchInnerKey: keypath
						as: bindingtype
						notifying: managedCollection
						with: keypath allButFirst.
					(managedCollection) 
							innerbind: (keypath allButFirst)
							withProperty: (keypath first) as: bindingtype.
					].
	]
	ifFalse: [
		"Wrong setup"
		Error new.
		]].! !

!Object methodsFor: '*bindings' stamp: 'AK 2/7/2010 19:41'!
innerunbind: keypath
"Unbinds all elements with that keypath internally"
	
	"Check if the key is already observed"
	(self bindings alreadyContainsKey: keypath) ifTrue: [
	
	"Check if name of exposed variable is inside this binder/bindee"
	(self class exposedBindings includes: keypath first)
		ifTrue: [
			
		self bindings allInnerBindings do: [ :binding |
				(binding localKeyPath == keypath) ifTrue: [
					binding target innerunbind: binding keyPath.
					self bindings
						stopObservingKey: binding localKeyPath
						regarding: binding target
						with: binding keyPath.
					self bindings allInnerBindings remove: binding.
					
					"Remove all ManagedCollections if it has no bindings any more"
					self removeManagedCollection: binding keyPath first
					]]]].! !

!Object methodsFor: '*bindings-support' stamp: 'topa 1/26/2010 10:37'!
instVarNamed: name ifAbsent: block

	^ self instVarAt: (self class
		instVarIndexFor: name asString
		ifAbsent: block)! !

!Object methodsFor: '*bindings-support' stamp: 'topa 2/9/2010 10:54'!
instVarNamedOrNil: name

	^ self instVarNamed: name ifAbsent: [^ nil]! !

!Object methodsFor: '*bindings' stamp: 'AK 1/31/2010 16:58'!
isBindable: bindable eligibleForKeyPath: aKeyPath
"checks if the keypath on this side references a collection and
if the bindable on the other side is a collection"

	^ (aKeyPath isCollection and:  [aKeyPath size > 1])
		and: [((self class exposedCollectionBindings includes: bindable) or: 
				[(self instVarNamedOrNil: bindable) isCollection])]
		and: [(aKeyPath second equals: 'arrangedObjects') or:
				[aKeyPath second equals: 'selection']]! !

!Object methodsFor: '*bindings' stamp: 'AK 2/5/2010 20:01'!
isBindableCollection: bindable
"checks if the keypath on this side references a complex value or collection"

	^ (bindable isSymbol and: [(self instVarNamedOrNil: bindable) isCollection])
	or: [(bindable isSymbol not and:  [bindable size = 1] and: [(self instVarNamedOrNil: bindable first) isCollection])]
	or: [(bindable isSymbol not and: [bindable size > 1])].! !

!Object methodsFor: '*bindings' stamp: 'AK 1/31/2010 12:03'!
isBindingDescriptor
"Only overwritten class objects that are descriptors will return true"

	^ false! !

!Object methodsFor: '*bindings' stamp: 'AK 1/31/2010 21:18'!
isManagedCollection
"Only overwritten class objects which have ManagedCollections will return true"

	^ false! !

!Object methodsFor: '*bindings' stamp: 'AK 2/7/2010 19:44'!
removeManagedCollection: bindable
"Removes the managed collection (unwrapping)"

	| bindableObject innerCollection |
	bindableObject := (self instVarNamed: bindable).
	
	(bindableObject isManagedCollection and:
	[bindableObject allInnerBindings isEmpty])
			ifTrue: [
				innerCollection := bindableObject actualCollection.
				bindableObject put: innerCollection.
				].

	^ innerCollection! !

!Object methodsFor: '*bindings' stamp: 'AK 2/6/2010 15:50'!
setVariableNamed: aSymbol toValue: anObject
"This method must be used in accessors to support the Bindings Coding Convention.
Additionally, the accessors must declare a <bindable> Pragma to completely fulfill Coding convention.
This method regularly sends update notifications to bound objects."
	
	| changedescriptor |
	
	changedescriptor := ChangeDescriptorMessage new
			operation: #set;
			keyPathOfChange: (Array with: aSymbol);
			lastSender: self;
			changedValue: anObject.
	
	(anObject = (self instVarNamed: aSymbol))
		ifFalse: [
			self bindings triggerEvent: #aboutToChange with: changedescriptor.
			self instVarNamed: aSymbol put: anObject.
			self bindings triggerEvent: #hasChanged with: changedescriptor].! !

!Object methodsFor: '*bindings' stamp: 'AK 2/6/2010 20:28'!
unbind: bindable from: anObject with: aKeyPath
"To unbind objects from another, the same arguments as in bind:to:with: must be used."

	| canonicalFormOfBindable canonicalFormOfKeypath foundDescriptors |
	canonicalFormOfBindable := self bringToCanonicalForm: bindable.

	canonicalFormOfKeypath := self bringToCanonicalForm: aKeyPath.
	foundDescriptors := self bindings observedBindings at: canonicalFormOfBindable ifAbsent: [OrderedCollection new].

	foundDescriptors ifNotEmpty: [
		"Check if the other side has already bound to the key"
		(foundDescriptors) do: [ :foundDescriptor |
			self innerunbind: bindable.
		].
		"Forget morph on the other side (with it's keypath)"
		(self bindings stopObservingKey: canonicalFormOfBindable regarding: anObject with: canonicalFormOfKeypath).
	
		"Init notification mechanism at other morph (object notifies morph notifies me)"
		anObject unbind: canonicalFormOfKeypath from: self with: canonicalFormOfBindable.
	].! !

!Object methodsFor: '*bindings' stamp: 'AK 2/6/2010 15:50'!
valueOfVariableNamed: aSymbol
"This method must be used in accessors to support the Bindings Coding Convention.
Additionally, the accessors must declare a <bindable> Pragma to completely fulfill Coding convention.
This method regularly sends access notifications to bound objects."

	| accessdescriptor |
	accessdescriptor := AccessDescriptorMessage new
			accessedKeyPath: (Array with: aSymbol);
			lastSender: self;
			accessedValue: (self instVarNamed: aSymbol).

	self bindings triggerEvent: #accessed with: accessdescriptor.
	^ accessdescriptor accessedValue.! !

OrderedCollection subclass: #BindingsTestCollection
	instanceVariableNames: 'a b'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsTestCollection methodsFor: 'binding methods' stamp: 'AK 1/6/2010 10:49'!
selectObject: anObject
      "The programmer has to expose this on binder side"
	self bindings triggerEvent: #objectSelected with: anObject.! !
